from time import time, sleep
from datetime import datetime, date

def call_sleep(seconds=0, minutes=0, hours=0, days=0, logger=None):
    message = f"Going to sleep for {days} days, {hours} hours, {minutes} minutes, {seconds} seconds"
    logger.info(message) if logger is not None else print(message)
    sleep((days * 24 * 60 * 60) + (hours * 60 * 60) + (minutes * 60) + seconds)

def change_date_format(date_string, fmt='%A %B %d, %Y'):
    d = datetime.strptime(date_string, fmt).date()
    return d.isoformat()

def get_days_difference(date_one, date_two=None, fmt='%Y-%m-%d'):
    t_d = date.today()  # Today date
    date_two = date_two if date_two is not None else t_d.isoformat()
    d = datetime.strptime(date_one, fmt) - datetime.strptime(date_two, fmt)
    return abs(d.days)

def time_measure(total_time):
    if total_time > 24 * 60 * 60:
        unit = 'days'
        total_time = total_time / 60 / 60 / 24

    elif total_time > 60 * 60:
        unit = 'hours'
        total_time = total_time / 60 / 60

    elif total_time > 60:
        unit = 'minutes'
        total_time = total_time / 60

    else:
        unit = 'seconds'

    return total_time, unit

def time_wrapper(func, logger=None):
    # Decorator that reports the execution time.
    def wrap(*args, **kwargs):
        start = time()
        result = func(*args, **kwargs)
        end = time()
        total_time, unit = time_measure(total_time=end - start)

        # Print | Log
        msg = f'This operation of {func.__name__} took {round(total_time, 2)} {unit}'
        if logger is not None:
            logger.debug(msg)
        else:
            print(msg)

        return result
    return wrap


from utility.db import DB
from pymongo import MongoClient
from pymongo.errors import ServerSelectionTimeoutError, DuplicateKeyError

class MongoDBHandler(DB):

    def __init__(self, name: str = ""):
        super(MongoDBHandler, self).__init__(name)
        
    def get_connection(self):
        try:
            self.client = MongoClient(self.db_uri)  # establish connection
            self.log('db establish connection')
        except ServerSelectionTimeoutError as _:
            message = 'db connection Timeout:\n' \
                      'For Cloud - check for if this machine ip is on whitelist\n' \
                      'For Local - check if the machine is running or if ports are blocked'
            self.logger.exception(message) if self.logger is not None else print(message)

    def get_db_names(self, key: str = 'name'):
        return [item[key] for item in self.client.list_databases()]

    @staticmethod
    def get_collections_names(db, key: str = 'name'):
        return [item[key] for item in db.list_collections()]

    @staticmethod
    def get_documents_list(collection, fil=None, sort='_id', ascending=1, limit=0, skip=0):
        fil = dict() if fil is None else fil
        return [item for item in collection.find(fil).sort(sort, ascending).limit(limit).skip(skip)]

    def get_db(self, name: str):
        db = self.client.get_database(name)
        self.log(f'got db: {name}')
        return db

    def get_collection(self, name: str, db):
        collection = db.get_collection(str(name))
        self.log(f'got collection: {name}')
        return collection

    def insert_document(self, collection, data: dict):
        ack = collection.insert_one(data)
        self.log('insert document to collection')
        return str(ack.inserted_id)

    def update_document(self, collection, fil: dict, data: dict):
        try:
            collection.update_one(fil, {"$set": data})
            self.log('updated document in the collection')
        except DuplicateKeyError:
            self.log('document is already in the collection')

    def is_document_exist(self, collection, fil: dict):
        cursor = collection.find(fil)
        try:
            cursor.next()
            flag = True
        except StopIteration:
            flag = False
        self.log(f'Document {"Not " * (not flag)}exist')
        return flag



from utility.config import Config
from utility.logger_handler import Logger

class DB():
    client = None

    def __init__(self, name: str):
        self.name = name
        self.db_uri = Config.db_uri
        self.logger = Logger(Config.name).get_logger()
        self.get_connection()

    def get_connection(self):
        raise NotImplementedError

    def log(self, message: str):
        if self.logger is not None:
            self.logger.debug(message)
        else:
            print(message)


from glob import glob
from pathlib import Path

class PathHandler():

    @staticmethod
    def get_path(path: str = None) -> str:
        path = str(Path.cwd()) if path is None else path
        path[-1].replace('\\', '')
        return path

    @staticmethod
    def drop_last_folder_from_path(path: str):
        return "\\".join(path.split('\\')[:-1])

    @staticmethod
    def create_dir(path: str):
        Path(path).mkdir(mode=511, parents=True, exist_ok=True)

    def get_files(self, folder_path: str = None, file_type='json') -> list[str]:
        path = self.get_path(folder_path)
        return [f for f in glob(path + "\*." + file_type)]



from utility.path_handler import PathHandler
from numpy import integer, floating, ndarray
from json import load, loads, dump, dumps, JSONEncoder, JSONDecodeError

class NpEncoder(JSONEncoder):
    def default(self, obj):
        if isinstance(obj, integer):
            return int(obj)
        elif isinstance(obj, floating):
            return float(obj)
        elif isinstance(obj, ndarray):
            return obj.tolist()
        else:
            return super(NpEncoder, self).default(obj)

class JsonHandler():
    cls_encoder: JSONEncoder = None
    file_encoder: str = 'utf-8'
    ensure_ascii: bool = True
    indent: int = 4

    def __init__(self, cls_encoder: str = None):
        self.setClsEncoder(cls_encoder)
        
    def setFileEncoder(self, file_encoder: str = 'utf-8'):
        # TODO add validator for correct input? like utf-8, utf-16, etc...
        self.file_encoder = file_encoder

    def setClsEncoder(self, cls_encoder: str = None):
        if cls_encoder == "NpEncoder":
                self.encoder = NpEncoder()

    def enableAscii(self, flag: bool = True):
        self.ensure_ascii = flag

    def setIndent(self, indent: int = 4):
        self.indent = indent

    def encode_data(self, data: dict):
        return loads(dumps(data, cls=self.cls_encoder))

    def save(self, data: dict, name: str = None, path: str = None):
        """Description: This save only localy"""
        name = 'NoName.json' if name is None else f"{name}.json"
        path = PathHandler().get_path(path)
        # File output
        with open(f"{path}\{name}", 'w') as outfile:
            data = data if type(data) is not object else data.decode(self.file_encoder)
            dump(data, outfile, indent=self.indent, ensure_ascii=self.ensure_ascii, cls=self.cls_encoder)
            print(f'File {name} was save on the path: {path}')

    def read(self, name: str, path: str = None):
        path = PathHandler().get_path(path)
        try:
            with open(f"{path}\{name}", encoding=self.file_encoder) as json_file:
                data = load(json_file)
            return data
        except JSONDecodeError as _:
            print(f'Error in decoding this file: {name}')
            return None

    def str_to_dict(self, data: str or list):
        if type(data) is str:
            return self.encode_data(data)
        elif type(data) is list:
            temp = list()
            for item in data:
                temp.append(self.str_to_dict(item))
            return temp



import logging
import logging.handlers
from utility.config import Config
from utility.path_handler import PathHandler
from cmreslogging.handlers import CMRESHandler

class Logger:
    logger: logging = None

    def __init__(self, name: str, path: str = None, logger=None, backupCount: int = 10):
        logging.raiseExceptions = Config.env == "Development"
        self.logger = self.start(name, path, logger, backupCount)

    def get_logger(self):
        return self.logger

    # input - logName as string, logPath as string, logger as logging class
    # output - logger as logging class
    # do - set logger settings
    @staticmethod
    def start(log_name: str = 'test', log_path: str = None, logger=None, backupCount: int = 10):
        name = log_name.replace('.log', '')
        path = log_path if log_path is not None else f"{PathHandler.get_path()}/logs"
        new_logger = logging.getLogger(name) if logger is None else logger
        
        new_logger.setLevel(logging.DEBUG)  # level per system status
        msg = '%(asctime)s %(levelname)s %(module)s %(name)s: %(message)s'
        formatter = logging.Formatter(msg, datefmt='%d-%m-%Y %H-%M-%S')
    
        # Stream Handler - cmd line
        stream_handler = logging.StreamHandler()
        stream_handler.setFormatter(formatter)

        # CMRES Handler - Elastic
        http_handler = CMRESHandler(hosts=[{'host': Config.elastic_url, 'port': Config.elastic_port}], es_index_name=name, 
                            auth_type=CMRESHandler.AuthType.NO_AUTH, es_additional_fields={'App': Config.name, 'Environment': Config.env})
        
        # File handler
        PathHandler.create_dir(path)
        file_handler = logging.handlers.RotatingFileHandler(f"{path}/{name}.log", maxBytes=10485760, 
                            backupCount=backupCount, encoding="UTF-8")
        file_handler.setFormatter(formatter)
        
        # adding Handlers
        new_logger.addHandler(file_handler)
        new_logger.addHandler(http_handler)
        new_logger.addHandler(stream_handler)

        new_logger.info('Initialize Logger')
        return new_logger



#!/usr/bin/env python3

class Error(Exception):
    """Base class for other exceptions"""
    pass


class PageNotLoaded(Error):
    """Exception raised for errors with status code of request.

        Attributes:
            url -- url of the request
            status_code -- status code of request
            message -- explanation of the error
        """

    def __init__(self, url, status_code, message=f"Got bad status code when trying to get the link provided"):
        self.url = url
        self.status_code = status_code
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f'Url: {self.url}\n' \
               f'Status Code: {self.status_code}\n' \
               f'Message: {self.message}\n' \
               f'Please try again later...\n' \
               f'Mean while you can check manually if the site is down or ip address is blocked.'


class ParseError(Error):
    """Exception raised for errors with status code of request.

        Attributes:
            url -- url of the request
            func -- function name where error accord
            message -- explanation of the error
        """

    def __init__(self, url, message=f"Got error when trying to parse the data provided"):
        self.url = url
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f'Url: {self.url}\n' \
               f'Message: {self.message}'



import requests
from utility.time_handler import call_sleep
from utility.exception_handler import PageNotLoaded


def connect(url, return_text: bool = True, return_json: bool = False, re_try: int = 5):
    r = requests.get(url)
    if r.status_code != 200:
        call_sleep(seconds=5)
        if re_try > 0:
            return connect(url, return_text, return_json, re_try=re_try - 1)
        raise PageNotLoaded(r.url, r.status_code)
    if return_text:
        return r.text
    if return_json:
        return r.json()


def post(url, data=None, return_text: bool = True, return_json: bool = False, re_try: int = 5):
    r = requests.post(url, data=data)
    if r.status_code != 200:
        call_sleep(seconds=5)
        if re_try > 0:
            return connect(url, return_text, return_json, re_try=re_try - 1)
        raise PageNotLoaded(r.url, r.status_code)
    if return_text:
        return r.text
    if return_json:
        return r.json()


from os import environ

class Config(object):
    env: str = environ.get('ENV') or "Development"
    name: str = environ.get('NAME')
    service_type: str = environ.get('TYPE')
    db_uri: str = environ.get('DB_URI')
    elastic_url: str = environ.get('Elastic_URL') or 'localhost'
    elastic_port: str = environ.get('ELASTIC_PORT') or 9200
